#!/usr/bin/env python
# -*- coding: utf-8 -*-
########################################################################
#
# tags.py
# Analyse der Tagzusammenhänge in einem WordPress-Blog.
#
# Dieses Skript wurde für das Datenbankdesign von WordPress 2.3.x
# erstellt, es sollte jedoch auch auf höheren Versionen laufen. Es
# setzt die Möglichkeit zum direkten Einloggen in die Datenbank
# voraus, was nicht auf jedem Webspace gegeben ist.
#
# Diese ist völlig freie Software, veröffentlicht unter Piratenlizenz
# http://tamagothi.wordpress.com/lizenz/
#
########################################################################


ID = "$Id: tags.py,v 1.4 2009/04/05 17:57:55 elias Exp $"


import sys
import MySQLdb


def analyze(dhost, duser, dpass, database, dprefix, limit=3, sublimit=2):
    con = MySQLdb.connect(host=dhost, user=duser, passwd=dpass,
                          db=database, use_unicode=True)
    # Um die Fummelei mit dem Präfix zu vereinfachen.
    termtbl = "%sterms" % (dprefix,)
    reltbl = "%sterm_relationships" % (dprefix,)
    taxtbl = "%sterm_taxonomy" % (dprefix,)
    # Ermitteln sämtlicher Tags
    tagsql = """\
    SELECT te.term_id, te.slug, tax.count
    FROM %s AS te
    JOIN %s AS tax
      ON te.term_id = tax.term_id
      AND taxonomy = 'post_tag'
    WHERE tax.count >= %d
    ORDER BY tax.count DESC, te.name
    """ % (termtbl, taxtbl, limit)
    # Ermitteln der Objekte, die ein bestimmtes Tag verwenden
    objsql = """\
    SELECT object_id
    FROM %s AS rel
    JOIN %s AS tax
      ON tax.term_taxonomy_id = rel.term_taxonomy_id
    JOIN %s AS te
      ON te.term_id = tax.term_id
    WHERE te.term_id = %%s
    """ % (reltbl, taxtbl, termtbl)
    termcur = con.cursor()
    termcur.execute(tagsql)
    for (term_id, term_name, count) in termcur.fetchall():
        print "%-38s %8d" % (term_name, count)
        # Wenn die verkackte MySQL Subselects koennte, wäre das einfacher!
        ocur = con.cursor()
        ocur.execute(objsql, (term_id,))
        objlist = ", ".join([str(i[0]) for i in ocur.fetchall()])
        ocur.close()
        relcur = con.cursor()
        relcur.execute("""\
        SELECT te.slug, COUNT(*) AS cnt
        FROM %s AS te
        JOIN %s AS tax
          ON tax.term_id = te.term_id
          AND tax.taxonomy = 'post_tag'
        JOIN %s AS rel
          ON rel.term_taxonomy_id = tax.term_taxonomy_id
        WHERE rel.object_id IN (%s)
        AND te.term_id != %%s
        GROUP BY te.name
        HAVING cnt >= %%s
        ORDER BY cnt DESC, te.name
        """ % (termtbl, taxtbl, reltbl, objlist), (term_id, sublimit))
        for (rterm_name, rcount) in relcur.fetchall():
            print "\t%-30s %8d" % (rterm_name, rcount)
        relcur.close()
    termcur.close()


def main():
    if len(sys.argv) not in (6, 7):
        print """\
Aufruf mit folgenden Parametern:

- Host        Der Datenbank-Host
- DB-User     Der Username für die Anmeldung
- DB-Passwort Das Passwort für die Anmeldung
- Datenbank   Der Name der Datenbank
- WP-Präfix   Das Präfix der WordPress-Tabellen
- Limit       opt. Mindesthäufigkeit der Tags (Vorgabe: 3)
- Sublimit    opt. Mindesthäufigkeit der mitbenutzten Tags (Vorgabe: 2)

Eine Verbindung zur Datenbank muss möglich sein, sonst kann dieses
Programm nicht arbeiten.
"""
        sys.exit("Fehlerhafter Aufruf")
    if len(sys.argv) > 8:
        sublimit = int(sys.argv[7])
    else:
        sublimit = 2
    if len(sys.argv) > 7:
        limit = int(sys.argv[6])
    else:
        limit = 3
    try:
        analyze(sys.argv[1], sys.argv[2], sys.argv[3],
                sys.argv[4], sys.argv[5], limit, sublimit)
    except:
        (type, value, tb) = sys.exc_info()
        sys.exit("Laufzeitfehler: %s" % (value[1], ))

if __name__ == '__main__':
    main()
